/*
 *  Name         : toplevel.cpp
 *  Aufgabe      : Toplevel-Widget von KHeisePro
 *  Version      : $Revision: 1.50 $
 *  Datum        : $Date: 1999/02/08 22:01:27 $
 *  Beschreibung : Komfortables Durchsuchen der Heise-Register-Datei
 *
 *  Copyright (C) 1998 Alexander Bednarz <A.Bednarz@gmx.de>
 *                     Martin Habbecke   <M.Habbecke@gmx.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */



#include <stdio.h>
#include <time.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>

#include <qmsgbox.h>
#include <qprogdlg.h>
#include <qstring.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qfont.h>
#include <qlist.h>
#include <qrect.h>
#include <qpopmenu.h>
#include <qfile.h>
#include <qfileinf.h>
#include <qdir.h>
#include <qdatetm.h>
#include <qclipboard.h>
#include <qregexp.h>

#include <kapp.h>
#include <ktopwidget.h>
#include <kmenubar.h>
#include <kpopmenu.h>
#include <ktoolbar.h>
#include <kstatusbar.h>
#include <kiconloader.h>

#ifdef HAVE_KKEYCONF_H
#include <kkeyconf.h>
#else
#include <kaccel.h>
#include <kkeydialog.h>
#endif /* HAVE_KKEYCONF_H */

#include <kfm.h>
#include <kprocess.h>
#include <kfiledialog.h>

#include "toplevel.h"
#include "database.h"
#include "mainwidget.h"
#include "dialogs.h"
#include "searchmask.h"
#include "found.h"




/* Konstruktor: 
 * Initialisierung: Toolbar und Statusbar angezeigt,
 *                  keine Database vorhanden.
 * Mainwidget, Menues, Toolbar und Statusbar anlegen */
TopLevel::TopLevel(const char *name = 0)
  : KTopLevelWidget(name)
{
  showToolbar = TRUE;
  showStatusbar = TRUE;
  database = 0;
  found = 0;
  actFound = 0;
  searchmask = 0;
  kfm = 0;
  clipSeparator = "| ";

  // Titel setzen
  setCaption( "KHeisePro - Suche im Heise-Register" );

  bindKeys();

  setupMenu();
  setupToolbar();
  setupStatusbar();
  updateMenuAccel();


  /* Mainwidget erzeugen und in TopLevel einsetzen */
  mainWidget = new MainWidget( this, "MainWidget" );
  setView( mainWidget );  

  /* Found-Vektor fuer die Vormerkung anlegen */
  markedFound = new Found;
  

  /* Connection: Durchgereichte headerClicked Signal mit
     dem Slot fuer dessen Behandlung */
  connect( mainWidget, SIGNAL(headerClicked(int)), 
	   this, SLOT(handleHeaderClicked(int)) );

  /* Connection: Wenn in der ListBox ein neuer Current
     ausgewaehlt wurde, wir das zugehoerige Entry an
     ShowEntry gereicht */
  connect( mainWidget, SIGNAL(currentChanged(int)),
	   this, SLOT(setNewEntryInShowEntry(int)) );

  /* Connection: Die Signale "printMarked()" und "printAll()" von
     MainWidget werden mit den Toplevel-Slotz verbunden */
  connect( mainWidget, SIGNAL(printMarked()), this, SLOT(printMarkedItems()) );
  connect( mainWidget, SIGNAL(printAll()), this, SLOT(printAllItems()) );
  connect( mainWidget, SIGNAL(exportMarked()), this, SLOT(exportMarkedItemsASCII()) );
  connect( mainWidget, SIGNAL(exportAll()), this, SLOT(exportAllItemsASCII()) );
  connect( mainWidget, SIGNAL(clipboardMarked()), this, SLOT(copyToClipboard()) );

  /* Connection: Die Signale fuer die Vormerkung werden mit den 
     passenden Slots von TopLevel verbundne */
  connect( mainWidget, SIGNAL(addAll()), this, SLOT(addAllToPreselect()) );
  connect( mainWidget, SIGNAL(addMarked()), this, SLOT(addMarkedToPreselect()) );
  connect( mainWidget, SIGNAL(removeAll()), this, SLOT(removeAllFromPreselect()) );
  connect( mainWidget, SIGNAL(removeMarked()), this, SLOT(removeMarkedFromPreselect()) );

  connect( mainWidget, SIGNAL(tabChanged(int)), this, SLOT(setNewTab(int)) );

  /* Dialogfeld fuer die Optionen erstellen */
  /* Muss global sein, damit die von kconfig eingelesenen
     Voreinstellungen erhalten bleiben */
  optionsDialog = new OptionsDialog( 0, "OptionsDlg" );

  /* Dialogfeld fuer die Suche erstellen */
  /* Glogales Dialogfeld, damit die Suchstrings zwischen den
     Suchen erhalten bleiben */
  searchDialog = new SearchDialog( 0, "SearchDlg" );

  /* Ok-Button des Dialogs mit Slot "startSearch" verbinden */
  connect( searchDialog, SIGNAL(STARTclicked()), this, SLOT(startSearch()) );

  /* Reset-Button des SearchDialogs mit Slot "sendSearchOpts" vom
     OptionsDialog verbinden */
  connect( searchDialog, SIGNAL(DEFAULTclicked()), optionsDialog, SLOT(sendSearchOpts()) );

  /* Signal "sendSearchOpts" von OptionsDlg mit Slot "receiveSearchOpts"
     vom SearchDlg verbinden */
  /* Wenn im SearchDialog auf "Reset" geklickt wird, wird das Signal
     RESETclicked an den Slot getSearchOpts vom OptinsDlg gesendet.
     Dieser Slot bereitet die Voreinstellungen in einem SearchMask-Objekt auf
     und sendet sie dann via "sendSearchOpts" Signal an den
     SearchDialog zurueck */
  connect( optionsDialog, SIGNAL(carrySearchOpts(SearchMask)),
	   searchDialog, SLOT(receiveSearchOpts(SearchMask)) );

  /* Connection: Wenn ein Default-Register im Options-Dialog eingetragen
     wurde und es gueltig ist, dann wird es auch sofort geoeffnet */
  connect( optionsDialog, SIGNAL(newDefaultRegister(const char *)),
	   this, SLOT(newRegister(const char *)) );

  /* Connection: Verbindung von OptionsDialog::supportCtrom(bool)
     mit dem Slot receiveSupportCtrom(bool) von MainWidget */
  connect( optionsDialog, SIGNAL(supportCtrom(bool)), 
	   this, SLOT(receiveSupportCtrom(bool)) );

  /* Connection: Wenn der Benutzer einen Artikel im KFM oeffnen will */
  connect( mainWidget, SIGNAL(openArticleClicked()), 
	   this, SLOT(openArticle()) );

  /* Connection: Durchreichen des Signals "doToleranceClicked" von SearchDialog
     an Mainwidget */
  connect( searchDialog, SIGNAL(doToleranceClicked(bool)),
	   mainWidget, SLOT(handleDoToleranceClicked(bool)) );


  /* OptionsDialog auffordern die Voreinstellungen zu laden */
  optionsDialog->loadOptions( 0 );

  /* Default-Werte in searchDialog uebernehmen */
  optionsDialog->sendSearchOpts();


  /* Default-Register oeffnen wenn eins eingetragen wurde*/
  //  setRegister( optionsDialog->getDefaultRegister() );

  
  /* Printer-PaintDevice erzeugen */
  printer = new QPrinter;

  printer->setOutputFileName( "kheisepro.ps" );
  printer->setOutputToFile( false );

  printer->setCreator( "KHeisePro" );

  /* testen, ob es Zeit fuer einen Update des Registers ist */
  if( optionsDialog->getAutoUpdate() )
    checkUpdateTime();
}



/* Destruktor: Toolbar muss "von Hand" geloescht werden */
TopLevel::~TopLevel()
{
  delete markedFound;
  delete ToolBar;
  delete kfm;
}



/* Erzeugen der Keyboard-Acceleratoren */
void TopLevel::bindKeys( void )
{
#ifndef HAVE_KKEYCONF_H
  /* So gehts mit dem 1.1-Branch... */
  Accel = new KAccel( this );
  
  // Items einfuegen
  Accel->insertStdItem( KAccel::Quit, "Beenden" );
  Accel->insertStdItem( KAccel::Open, "Register ffnen" );
  Accel->insertStdItem( KAccel::Close, "Register schlieen" );
  Accel->insertStdItem( KAccel::Find, "Suchen" );
  Accel->insertStdItem( KAccel::Help, "Hilfe" );
  Accel->insertItem( "Alle Eintrge vormerken", "markall", "CTRL+A" );
  Accel->insertItem( "Markierte Eintrge vormerken", "markmarked", "CTRL+M" );

  // Aktionen mit den Slots verbinden 
  Accel->connectItem( KAccel::Quit, this, SLOT(exitProgram()) );
  Accel->connectItem( KAccel::Open, this, SLOT(openRegister()) );
  Accel->connectItem( KAccel::Close, this, SLOT(closeRegister()) );
  Accel->connectItem( KAccel::Find, this, SLOT(searchDlg()) );
  Accel->connectItem( KAccel::Help, this, SLOT(help()) );
  Accel->connectItem( "markall", this, SLOT(addAllToPreselect()) );
  Accel->connectItem( "markmarked", this, SLOT(addMarkedToPreselect()) );

  Accel->readSettings();

#else
  /* ... und so mit 1.0 */
  /* Acceleratoren fuer die Menues erzeugen */
  initKKeyConfig( kapp->getConfig() );

  // create functionName/keyCode association
  kKeys->addKey( "Beenden", "CTRL+Q" );
  kKeys->addKey( "Register ffnen", "CTRL+O" );
  kKeys->addKey( "Register schlieen", "CTRL+W" );
  kKeys->addKey( "Suchen", "CTRL+F" );
  kKeys->addKey( "Hilfe", "F1" );
  kKeys->addKey( "Alle Eintrge vormerken", "CTRL+A" );
  kKeys->addKey( "Markierte Eintrge vormerken", "CTRL+M" );
  
  QString widgetName = this->name();

  kKeys->registerWidget( widgetName, this);

  kKeys->connectFunction( widgetName, "Beenden", this, SLOT(exitProgram()) );
  kKeys->connectFunction( widgetName, "Register ffnen", this, SLOT(openRegister()) );
  kKeys->connectFunction( widgetName, "Register schlieen", this, SLOT(closeRegister()) );
  kKeys->connectFunction( widgetName, "Suchen", this, SLOT(searchDlg()) );
  kKeys->connectFunction( widgetName, "Hilfe", this, SLOT(help()) );
  kKeys->connectFunction( widgetName, "Alle Eintrge vormerken", this, SLOT(addAllToPreselect()) );
  kKeys->connectFunction( widgetName, "Markierte Eintrge vormerken", this, SLOT(addMarkedToPreselect()) );
#endif /* HAVE_KKEYCONF_H */
}



/* Hinzufuegen / Aendern der Menuepunkte
   -> Acceleratoren werden mit angezeigt */
void TopLevel::updateMenuAccel( void )
{
#ifndef HAVE_KKEYCONF_H
  Accel->changeMenuAccel( m_File, exitID, KAccel::Quit );
  Accel->changeMenuAccel( m_File, openID, KAccel::Open );
  Accel->changeMenuAccel( m_File, closeID, KAccel::Close );
  Accel->changeMenuAccel( m_Search, searchID, KAccel::Find );
  Accel->changeMenuAccel( m_Help, helpID, KAccel::Help );
  Accel->changeMenuAccel( m_Search, preAllID, "markall" );
  Accel->changeMenuAccel( m_Search, preMarkedID, "markmarked" );

#else
  changeMenuAccel( m_File, exitID, "Beenden" );
  changeMenuAccel( m_File, openID, "Register oeffnen" );
  changeMenuAccel( m_File, closeID, "Register schlieen" );
  changeMenuAccel( m_Search, searchID, "Suchen" );
  changeMenuAccel( m_Help, helpID, "Hilfe" );
  changeMenuAccel( m_Search, preAllID, "Alle Eintrge vormerken" );
  changeMenuAccel( m_Search, preMarkedID, "Markierte Eintrge vormerken" );
#endif /* HAVE_KKEYCONF_H */
}


#ifdef HAVE_KKEYCONF_H
/* Zu einem Menuepunkt den Accelerator hinzufuegen */
/* Betrifft nur den Text, der im Menue steht */
void TopLevel::changeMenuAccel ( QPopupMenu *menu, int id,
				 const char *functionName )
{
  QString s = menu->text( id );
  if ( !s ) return;
  
  int i = s.find('\t');
  
  QString k = keyToString( kKeys->readCurrentKey( functionName ) );
  if( !k ) return;
  
  if ( i >= 0 )
    s.replace( i+1, s.length()-i, k );
  else {
    s += '\t';
    s += k;
  }
  
  menu->changeItem( s, id );
}
#endif /* HAVE_KKEYCONF_H */


/* Menues anlegen */
void TopLevel::setupMenu(void)
{
  /* Items des Menubars */
  /* Menu: "Datei" */
  m_File = new QPopupMenu;
  CHECK_PTR(m_File);
  openID = m_File->insertItem("Register ffnen...", this, SLOT(openRegister()));
  closeID = m_File->insertItem( "Register schlieen", this, SLOT(closeRegister()) );
  updateID = m_File->insertItem( "Register aktualisieren...", this, SLOT(updateRegister()) );
  appendID = m_File->insertItem("Daten anhngen...", this, SLOT(appendData()));
  m_File->insertSeparator();


  /* Untermenu "ASCII-Export" */
  m_Ascii = new QPopupMenu;
  CHECK_PTR(m_Ascii);
  exportAllID = m_Ascii->insertItem( "Gefundene Eintrge...", this, SLOT(exportMenuAllItems()));
  exportPreID = m_Ascii->insertItem( "Vorgemerkte Eintrge...", this, SLOT(exportMenuPreItems()));
  m_File->insertItem( "ASCII-Export", m_Ascii );
  /* Eintraege des Exports disablen, man kann nur exportieren, wenn ein
     Found vorhanden ist */
  m_File->setItemEnabled( exportAllID, false );
  m_File->setItemEnabled( exportPreID, false );


  /* Untermenu "Drucken" */
  m_Print = new QPopupMenu;
  CHECK_PTR(m_Print);
  printConfID = m_Print->insertItem( "Drucker...", this, SLOT(printerConfigure()) );
  printAllID = m_Print->insertItem("Gefundene Eintrge", this, SLOT(printMenuAllItems()));
  printPreID = m_Print->insertItem("Vorgemerkte Eintrge", this, SLOT(printMenuPreItems()));
  m_File->insertItem ("Drucken", m_Print);
  m_File->insertSeparator();
  /* Eintraege fuers Drucken disablen, man kann nur drucken, wenn ein 
     Found vorhanden ist */
  m_File->setItemEnabled( printAllID, false );
  m_File->setItemEnabled( printPreID, false );

  exitID = m_File->insertItem( "Beenden", this, SLOT(exitProgram()));


  /* Menu: "Suchen" */
  m_Search = new QPopupMenu;
  CHECK_PTR(m_Search);
  searchID = m_Search->insertItem( "Suche...", this, SLOT(searchDlg()) );
  m_Search->insertSeparator();
  /* Menuepunkte fuer die Vormerkungen */
  preAllID = m_Search->insertItem( "Alle Eintrge vormerken" , this, SLOT(addAllToPreselect()) );
  preMarkedID = m_Search->insertItem( "Markierte Eintrge vormerken", this, SLOT(addMarkedToPreselect()) );
  remAllID = m_Search->insertItem( "Alle Eintrge entfernen", this, SLOT(removeAllFromPreselect()) );
  remMarkedID = m_Search->insertItem( "Markierte Eintrge entfernen", this, SLOT(removeMarkedFromPreselect()) );



  /* Menu: "Einstellungen" */
  m_Options = new QPopupMenu;
  CHECK_PTR(m_Options);
  m_Options->setCheckable( TRUE ); // Im Options-Menu sind Check-Items moeglich

  toolbarID = m_Options->insertItem("Toolbar", this, SLOT(toggleToolbar()));
  m_Options->setItemChecked( toolbarID, TRUE );  // Toolbar ist zu Anfang angezeigt

  statusbarID = m_Options->insertItem("Statusbar", this, SLOT(toggleStatusbar()));
  m_Options->setItemChecked( statusbarID, TRUE );  // Statusbar ist zu Anfang angezeigt

  m_Options->insertSeparator();
  optionsID = m_Options->insertItem( "Voreinstellungen...", this, SLOT(showOptions()) );
  confkeysID = m_Options->insertItem( "Tastenbelegung...", this, SLOT(configureKeyboard()) );

  /* Menu: "Help" */
  m_Help = new QPopupMenu;
  CHECK_PTR( m_Help );
  helpID = m_Help->insertItem( "Hilfe...", this, SLOT(help()) );
  m_Help->insertSeparator();
  aboutID = m_Help->insertItem( "ber...", this, SLOT(about()) );

  /* Menubar anlegen und Menues eintragen */
  MenuBar = new KMenuBar(this);
  CHECK_PTR(MenuBar);
  MenuBar->insertItem( "Datei", m_File );
  MenuBar->insertItem( "Suchen", m_Search );
  MenuBar->insertItem( "Einstellungen", m_Options );
  MenuBar->insertSeparator();  // Help-Menu soll ganz nach rechts
  MenuBar->insertItem( "Hilfe", m_Help );
  setMenu(MenuBar);
}



/* Toolbar anlegen, alle Buttons erzeugen 
   und mit den Slots verbinden */
void TopLevel::setupToolbar(void)
{
  // Speicher fuer ein Icon
  QPixmap pm;
  // Loader fuer die Button-Icons
  KIconLoader *loader = kapp->getIconLoader();

  /* Toolbar anlegen */
  ToolBar = new KToolBar(this, "KheiseProToolbar");
  CHECK_PTR(ToolBar);

  /* Icons laden und Buttons anlegen */
  pm = loader->loadIcon( "fileopen.xpm" );
  ToolBar->insertButton(pm, 0, SIGNAL( clicked() ), this,
			SLOT( openRegister() ), TRUE, "Register ffnen" );

  pm = loader->loadIcon( "fileclose.xpm" );
  ToolBar->insertButton( pm, 1, SIGNAL(clicked()), this,
			 SLOT( closeRegister() ), TRUE, "Register schlieen" );

  pm = loader->loadIcon( "exit.xpm" );
  ToolBar->insertButton( pm, 2, SIGNAL( clicked() ), this, 
			 SLOT( exitProgram() ), TRUE, "Exit" );

  ToolBar->insertSeparator();

  pm = loader->loadIcon( "editcopy.xpm" );
  ToolBar->insertButton( pm, 3, SIGNAL( clicked() ), this,
			 SLOT( copyToClipboard() ), TRUE, 
			 "Markierte Eintraege in Clipboard kopieren" );

  ToolBar->insertSeparator();

  pm = loader->loadIcon( "search.xpm" );
  ToolBar->insertButton( pm, 4, SIGNAL( clicked() ), this,
			 SLOT( searchDlg() ), TRUE, "Suchen" );

  ToolBar->insertSeparator();

  pm = loader->loadIcon( "configure.xpm" );
  ToolBar->insertButton( pm, 5, SIGNAL( clicked() ),this,
			 SLOT( showOptions() ), TRUE, "Voreinstellungen" );
  ToolBar->setBarPos(KToolBar::Top);
  addToolBar(ToolBar);
}



/* Erzeugen und unterteilen des Statusbars */
void TopLevel::setupStatusbar(void)
{
  /* StatusBar anlegen */
  StatusBar = new KStatusBar(this, "KheiseProStatus");

  /* Statusbar muss unterteilt werden */
  StatusBar->insertItem("Eintrge:        ", 1);
  StatusBar->insertItem("Register: (kein)", 2);


  StatusBar->enable(KStatusBar::Show);
  setStatusBar(StatusBar);
}



/* Funktions zur Ueberpruefung, ob das Zeitintervall fuer einen
   Register-Update abgelaufen ist */
void TopLevel::checkUpdateTime( void )
{
  /* KConfig holen */
  KConfig *config = kapp->getConfig();

  config->setGroup( "Register" );
  QString entry = config->readEntry( "LetzterUpdate" );

  if( !entry.isEmpty() )
    {
      int day = entry.mid( 0, 2 ).toInt();
      int month = entry.mid( 3, 2 ).toInt();
      int year = entry.mid( 6, 4 ).toInt();
      //      cout << "last update-time: " << day << "." << month << "." << year << endl;
      QDate lastUpdate( year, month, day );
      QDate today = QDate::currentDate();

      /* Ist ein Update noetig ?? */
      //      cout << "vergangene Tage: " << lastUpdate.daysTo( today ) << endl;
      if( lastUpdate.daysTo( today ) >= optionsDialog->getUpdateIntervall() )
	/* Es ist !! */
	if( !QMessageBox::information( this, "kHeisePro - Register Update",
				       "Der eingestellte Update-Zeitraum ist berschritten.\n"
				       "Soll eine neue Registerdatei via FTP bezogen werden ?",
				       "Ja", "Nein" ) )
	  updateRegister();
    }
  else
    {
      if( QMessageBox::information( this, "kHeisePro - Register Update",
				    "In der Konfigurations-Datei wurde kein Datum des\nletzten "
				    "Register-Updates gefunden.\nSoll jetzt ein Register-Update "
				    "durchgefhrt werden ?", "Ja", "Nein" ) == 0 )
	{
	  updateRegister();
	}
    }
}



/*********************************************
 * Slots der Menupunkte
 *********************************************/

/* pruefen des uebergebenen Filenamens auf Gueltigkeit, evtl.
   Frage an den User, ob er offene Datei schliessen will, dann
   setzen des neuen Registers */
void TopLevel::newRegister( const char *fileName )
{
  
  if ( !Database::isValid( fileName ) ) {
    // keine gueltige Registerdatei!
    // -> Fehlermeldung
    
    QMessageBox::warning( this, "KHeisePro", 
			  "Es wurde eine ungltige Registerdatei ausgewhlt!" );
    return;
  }

  /* Wenn schon eine Register-Datei offen ist
     fragen, ob diese geschlossen werden soll und dann evtl.
     neuen Dateinamen auswaehlen lassen */
  if ( database )
    {
      if( QMessageBox::warning( this, "KHeisePro", 
				"Aktuelle Registerdatei wird geschlossen\n"
				"Gefundene Eintrge gehen verloren !\n", 
				QMessageBox::Ok | QMessageBox::Default,
				QMessageBox::Cancel | QMessageBox::Escape )
	  == QMessageBox::Cancel )
	/* Wenn "Cancel" gewaehlt wurde */
	return;
      else
	{
	  /* Wenn "Ok" gewaehlt wurde */
	  closeRegister();
	  // found loeschen;
	}
    }

  /* Filenamen setzen */
  setRegister( fileName );
}



/* Behandlung des Menuepunkts: Datei->Register Oeffnen... */
void TopLevel::openRegister()
{

  // neuen Dateinamen holen
  // Filter
  QString fileName = KFileDialog::getOpenFileName( 0, "*.frm *.FRM"
						   " *.cti *.CTI | Registerdatei" );

  if ( fileName.isEmpty() )
    return; // Benutzer wollte doch nicht 

  newRegister( fileName );
}



/* Slot zum setzen des Dateinames der Datendatei
   => Anlegen von Database */
// filename ist auf jeden Fall nicht leer
// filename ist auf jeden Fall gueltig
void TopLevel::setRegister( const QString & filename )
{
  // cout << "setting register" << endl;

  /* Database mit Filenamen anlegen */

  // Der Eintrag in der Config-Datei war falsch
  if ( filename.isEmpty() )
    return;

  //  if( Database::isValid( filename ) )
  //    {
  database = new Database( filename );
  
  /* Name der Datei in Statuszeile eintragen */
  QString hlp( "Register: " );
  hlp.append( filename );
  StatusBar->changeItem( (const char *) hlp , 2 );
  
  /* Jetzt kann man suchen */
  searchDialog->enableStartBut( true );
  
  /* Man kann auch Daten an die Registerdatei anhaengen */
  m_File->setItemEnabled( appendID, true );

  // }
  //  else
  //    {
      
      /* Man kann nicht suchen... */
  //      searchDialog->enableStartBut( false );
      
      /* ... und Daten an die Registerdatei anhaengen */
  //      m_File->setItemEnabled( appendID, false );

  //      StatusBar->changeItem( "Register: (kein)", 2 );
  //    }

}


/* Behandlung des Menuepunkts: Datei->Register schliessen */
/* Schliessen einer ex. Registerdatei */
void TopLevel::closeRegister()
{
  /* Ist ueberhaupt eine Database anelegt worden ? */
  if (database)
    {
      /* Wenn ja, dann diese loeschen */
      delete database;
      database = 0;
      
      /* Gibts es einen Found ?? */
      if( found )
	{
	  delete found;
	  found = 0;
	  
	  /* Es gibt keinen Found => man kann nicht drucken und nicht 
	     vormerken */
	  m_File->setItemEnabled( printAllID, false );
	  //	  m_File->setItemEnabled( printActualID, false );
	  m_Search->setItemEnabled( preAllID, false );
	  m_Search->setItemEnabled( preMarkedID, false );
	}
      //      mainWidget->updateShowEntry( 0 );

      /* man kann nicht mehr suchen => Start-Button disablen */
      searchDialog->enableStartBut( false );

      /* man kann keine Daten an die Registerdatei anhaengen */
      m_File->setItemEnabled( appendID, false );

      /* Muss der aktuelle Tab geupdatet werden ? */
      if( mainWidget->getActTab() == 0 )
	setNewTab( mainWidget->getActTab() );
    }
  
  /* Registernamen aus Statuszeile loeschen */
  StatusBar->changeItem( "Register: (kein)", 2 );
  StatusBar->changeItem( "Eintrge:", 1 );
}



/* Behandlung des Menuepunkts: Datei->Register-update */
/* Holen einer aktuellen Registerdatei via FTP */
void TopLevel::updateRegister( void )
{
  if( kfm == 0 )
    {
      kfm = new KFM;
      // cout << "starting new KFM" << endl;
      kfm->allowKFMRestart( true );

      /* Ende-Signal von KFM-copy mit Slot unzipRegister verbinden */
      connect( kfm, SIGNAL(finished()), this, SLOT(unzipRegister()) );
    }

  tmpfile = new char[L_tmpnam + 2];
  tmpnam( tmpfile );
  QString srcfile;
  srcfile = optionsDialog->getUpdateURL();
  
  /* Kopieren starten */
  kfm->copy( srcfile, tmpfile );
}



/* Slot: entpacken der RegisterDatei */
void TopLevel::unzipRegister( void )
{
  /* Verzeichnis des Tmpfiles herausfinde */
  QDir tmpDir( QFileInfo( tmpfile ).dir( true ) );

  KProcess *process = new KProcess;

  /* Argumente an KProcess uebergeben */
  QDir pwd = QDir::current(); // current work dir sichern
  QDir::setCurrent( tmpDir.path() );
  (*process) << "unzip" << "-o" << tmpfile << "INHALT.FRM";
  // cout << "KProcess: " << endl;
  process->start( KProcess::Block );
  QDir::setCurrent( pwd.path() );  // current work dir wieder herstellen

  /* Wenn das entpacken geklappt hat .. */
  bool goon = false;
  char frmName[11];
  /* Wie mag die entpackte Datei heissen ??? :-) */
  if( tmpDir.exists( "inhalt.frm" ) )
    {
      strcpy( frmName, "inhalt.frm" );
      goon = true;
    }
  if( tmpDir.exists( "INHALT.FRM" ) )
    {
      strcpy( frmName, "INHALT.FRM" );
      goon = true;
    }

  if( goon )
    {
      QString srcName = tmpDir.filePath( frmName, true );

      QString defaultName = optionsDialog->getDefaultRegister();
      defaultName.remove( defaultName.findRev( '/' ), defaultName.length() );
      QString saveName;
  
      bool done1 = false;
      bool done2 = false;
      do {
	saveName = KFileDialog::getSaveFileName( defaultName, "*.frm *.FRM | Registerdatei" );
	if( !saveName.isEmpty() )
	  {
	    /* Kann die Registerdatei verschoben werden ? */
	    if( !(done1 = moveFile( srcName, saveName )) )
	      /* Es hat nicht geklappt */
	      QMessageBox::information( this, "kHeisePro", "Die Register-Datei konnte nicht an\n"
					"die angegebene Stelle verschoben werden.\n\n"
					"Bitte einen anderen Dateinamen auswhlen." );
	    else
	      {
		/* Es hat geklappt */
		/* Datum des letzten Updates speichern */
		/* KConfig holen */
		KConfig *config = kapp->getConfig();
		
		config->setGroup( "Register" );      
		QString date;
		QDate today = QDate::currentDate();
		date.sprintf( "%02d.%02d.%04d", today.day(), today.month(), today.year() );
		config->writeEntry( "LetzterUpdate", date );
		
		/* Erfolgsmeldung ausgeben */
		QString mesg;
		mesg.sprintf( "Eine Registerdatei wurde erfolgreich von\n\n%s\n\ngeholt, entpackt, "
			      "und nach\n\n%s\n\nverschoben.", optionsDialog->getUpdateURL(), (const char *) saveName );
		QMessageBox::information( this, "kHeisePro", mesg );
	      }
	  }
	else
	  {
	    done2 = ( QMessageBox::information( this, "kHeisePro", "Es wurde kein Ziel-Dateiname fr "
						"die Registerdatei\n"
						"ausgewhlt.\n\nSoll die neue Registerdatei wirklich "
						"gelscht werden ?", "Ja", "Nein" ) == 0 );
	  }
      
      } while( !done1 && !done2  );

    }
  else
    QMessageBox::information( this, "kHeisePro", "Die Registerdatei konnte nicht\n"
			      "ordnungsgem kopiert / entpackt werden !" );

  /* Moeglicherweise uebrig gebliebenen Tmp-Dateien loeschen */
  tmpDir.remove( frmName );
  tmpDir.remove( tmpfile );
  delete [] tmpfile;
  delete process;
}



/* Verschieben des Files ueber File-System-Grenzen hinaus */
/**********************************************************************/
/* extrem anfaellige implementation... sehr verbesserungswuerdig !!! */
/**********************************************************************/
bool TopLevel::moveFile( const char * from, const char * to )
{
  KProcess *process = new KProcess;
  (*process) << "mv" << from << to;

  QFile testFile( to );
  /* testen, ob auf das angegebene File geschrieben werden kann */
  if( testFile.open( IO_WriteOnly ) )
    {
      testFile.close();
      process->start( KProcess::Block );
      return true;
    }
  else
    {
      return false;
    }
}




/* Behandlung des Menuepunkts: Datei->Daten anhaengen... */
/* Anhaengen einer Update-Datei an eine geoeffnete Registerdatei */
void TopLevel::appendData( void )
{
  /* Ist eine Registerdatei offen ? */
  //  if ( !database )
  //  {
      /* Wenn nicht, dann Meldung ausgeben und abbrechen */
  //     QMessageBox::information( this, "KHeisePro", 
  //				"Es wurde keine Registerdatei geoeffnet.\n" );
  //     return;
  //  }
  
  /* Wenn ja, dann nach dem Namen der Update-Datei fragen
     und damit database->append aufrufen */
  // Filter setzen
  QString appendName = KFileDialog::getOpenFileName( 0, "*.cti *.CTI | CTI - Files" );
  if ( !appendName.isEmpty() )
    database->append( appendName);
}




/* Behandlung des Menuepunkts: Datei->Drucken->Drucker... */
void TopLevel::printerConfigure( void )
{
  printer->setup(0);
}




/* Behandlung des Menuepunkts: Datei->ASCII-Export->Alle Eintraege */
void TopLevel::exportMenuAllItems( void )
{
  int ret = exportFoundASCII( found );
  if( ret == 0 )
    QMessageBox::information( this, "KHeisePro - ASCII-Export",
			      "Alle gefundenen Eintraege wurden\n"
			      "erfolgreich exportiert!" );
  else if( ret == 2 )
    QMessageBox::warning( this, "KHeisePro - ASCII-Export", 
			  "Ausgewaehlte Datei konnte nicht geoeffnet werden!" );
}


/* Behandlung des Menuepunkts: Datei->ASCII-Export->Vorgemerkte Eintraege */
void TopLevel::exportMenuPreItems( void )
{
  int ret = exportFoundASCII( markedFound );
  if( ret == 0 )
    QMessageBox::information( this, "KHeisePro - ASCII-Export",
			      "Alle vorgemerkten Eintraege wurden\n"
			      "erfolgreich exportiert!" );
  else if( ret == 2 )
    QMessageBox::warning( this, "KHeisePro - ASCII-Export", 
			  "Ausgewaehlte Datei konnte nicht\ngeoeffnet werden!" );
}


/* Exportieren ALLER aktuellen Eintraege */
void TopLevel::exportAllItemsASCII( void )
{
  int ret = exportFoundASCII( actFound );
  if( ret == 0 )
    QMessageBox::information( this, "KHeisePro - ASCII-Export",
			      "Alle aktuellen Eintraege wurden\n"
			      "erfolgreich exportiert!" );
  else if( ret == 2 )
    QMessageBox::warning( this, "KHeisePro - ASCII-Export", 
			  "Ausgewaehlte Datei konnte nicht geoeffnet werden!" );
}


/* Exportieren nur der MARKIERTEN aktuellen Eintraege */
void TopLevel::exportMarkedItemsASCII( void )
{
  int ret;
  QList<int> markList = mainWidget->getMarkedItems();

  Found *expVek = new Found;
  int *index = markList.first();
  do
    {
      actFound->setIndex( *index );
      expVek->append( *actFound->get() );
    } while( (index = markList.next()) );

  ret = exportFoundASCII( expVek );
  if( ret == 0 )
    QMessageBox::information( this, "KHeisePro - ASCII-Export",
			      "Alle markierten Eintraege wurden\n"
			      "erfolgreich exportiert!" );
  else if( ret == 2 )
    QMessageBox::warning( this, "KHeisePro - ASCII-Export", 
			  "Ausgewaehlte Datei konnte nicht geoeffnet werden!" );

  delete expVek;
}



/* Behandlung des Menuepunkts: Datei->Drucken->Alle Eintraege */
void TopLevel::printMenuAllItems( void )
{
  // den actFound sichern
  Found *save = actFound;
  actFound = found;

  // Drucken des actFound
  printAllItems();

  // actFound wieder herstellen
  actFound = save;
}



/* Behandlung des Menuepunkts: Datei->Drucken->Alle vorgemerkten Eintraege */
void TopLevel::printMenuPreItems( void )
{
  // den actFound sichern
  Found *save = actFound;
  actFound = markedFound;

  // Drucken des actFound
  printAllItems();

  // actFound wieder herstellen
  actFound = save;  
}



/* Drucken ALLER Eintraege im actFound */
void TopLevel::printAllItems( void )
{
  printFound( actFound );
  QMessageBox::information( this, "KHeisePro", 
			    "Liste der Eintrge wurde gedruckt." );
}


/* Drucken der MARKIERTEN Eintraege im actFound */
void TopLevel::printMarkedItems( void )
{
  QList<int> markList = mainWidget->getMarkedItems();

  Found *printVek = new Found;
  int *index = markList.first();
  do
    {
      actFound->setIndex( *index );
      printVek->append( *actFound->get() );
    } while( (index = markList.next()) );
  printFound( printVek );

  delete printVek;
  QMessageBox::information( this, "KHeisePro", 
			    "Liste der markierten Eintrge\nwurde gedruckt." );
}



/* Behandlung des Menuepunkts: Datei->Beenden */
void TopLevel::exitProgram( void )
{
  // Voreinstellungen von Beenden speichern
  optionsDialog->saveOpts();
  kapp->quit();
}



/* Behandlung des Menuepunkts: Suchen->Suche... */
void TopLevel::searchDlg( void )
{
  /* Ist eine Registerdatei offen ? */
  //  if ( !database )
  // {
      /* Wenn nicht, dann Meldung ausgeben und abbrechen */
      //      QMessageBox::information( this, "KHeisePro", 
      //				"Es wurde keine Registerdatei geoeffnet\n" );
      //      return;
  //    }
  searchDialog->show();
  searchDialog->raise();
}


/* Starten einer festgelegten Suche */
void TopLevel::startSearch( void )
{
  /* Testen, ob die Angaben im SuchDialog ausreichend sind */
  if( !searchDialog->checkConsistency() )
    return;

  /* Wurde eine Registerdatei geoeffnet ? */
  //  if ( !database )
  //  {
      /* Wenn nicht, dann Meldung ausgeben und abbrechen */
  //    QMessageBox::information( this, "KHeisePro", 
  //    				"Es wurde keine Registerdatei geoeffnet.\n" );
  //    return;
  //  }

  // cout << "starting search" << endl;

  /* Found fuer die neue Suche anlegen */
  Found *newFound = new Found;

  /* SearchMask anlegen und mit Infos aus dem
     SearchDialog auffuellen */
  searchmask = new SearchMask;
  searchDialog->getContent( *searchmask );
  //  searchDialog->hide();
  //  kapp->processEvents();

  // cout << "doing search" << endl;

  /* Prgress-Dialog erzeugen */
  QProgressDialog PrgDlg( "Suche luft...", "Abbruch", 100, 0, 0, true);

  /* Connections zwischen ProgressDialog <=> Database */
  connect( database, SIGNAL(databasePosition( int )), &PrgDlg, SLOT(setProgress(int)) );
  connect( &PrgDlg, SIGNAL(cancelled()), database, SLOT(abortSearch()) );
  //  PrgDlg.reset();
  //  PrgDlg.update();
  PrgDlg.show();

  /* Suche durchfueren */
  int foundanz = database->search( newFound, *searchmask );
  
  /* Wurde die Suche abgebrochen ?? */
  if( !PrgDlg.wasCancelled() )
    {
      /* Wenn nicht, dann alten Found durch neuen ersetzen */
      delete found;
      found = newFound;

      /* Wurden Eintraege gefunden ?? */
      if( foundanz > 0 )
	{
	  /* ListBox mit neuem Found auffuellen */
	  //	  mainWidget->fillList( found );
	  /* StatusBar mit neuen Werten fuellen */
	  QString hlp( "Eintrge: " );
	  hlp.append( QString().setNum( foundanz ) );
	  StatusBar->changeItem( (const char *) hlp, 1 );

	  /* Man kann drucken */
	  m_File->setItemEnabled( printAllID, true );
	  //	  m_File->setItemEnabled( printActualID, true );
	  /* Man kann eintraege in das Clipboard kopieren */
	  m_File->setItemEnabled( exportAllID, true );
	  /* Hauptfenster nach vorne holen */
	  raise();
	}
      else
	{
	  /* Inhalt von ListBox und ShowEntry loeschen */
	  //mainWidget->fillList( 0 );
	  //mainWidget->updateShowEntry( 0 );
	  /* StatusBar updaten */
	  StatusBar->changeItem( "Eintrge: 0", 1 );
	  /* Found wieder loeschen */
	  delete found;
	  found = 0;
	  
	  /* Man kann nicht drucken und vormerken */
	  m_File->setItemEnabled( printAllID, false );
	  /* Man kann keine Eintraege in das Clipboard kopieren */
	  m_File->setItemEnabled( exportAllID, false );
	  //	  m_File->setItemEnabled( printActualID, false );
	  //	  m_Search->setItemEnabled( preAllID, false );
	  //	  m_Search->setItemEnabled( preMarkedID, false );
	}
      /* Auf den Tab der Suche Wechseln */
      if( mainWidget->getActTab() == 0 )
	//  nur Anzeige des neuen Found noetig 
	setNewTab( 0 );
      else
	// Wechsel auf den Such-Tab
	mainWidget->setActTab( 0 );

    }
  else
    /* Suche wurde abgebrochen */
    delete newFound;

  PrgDlg.reset();

  delete searchmask;
}



/* Behandlung des Menuepunkts: Optionen->Voreinstellungen... */
void TopLevel::showOptions( void )
{
  optionsDialog->show();
  optionsDialog->raise();
}



/* Behandlung des Menuepunkts: Options->Tastenbelegung */
void TopLevel::configureKeyboard( void )
{
#ifndef HAVE_KKEYCONF_H
  KKeyDialog::configureKeys( Accel );
#else
  kKeys->configureKeys( this );
#endif /* HAVE_KKEYCONF_H */

  updateMenuAccel();
  mainWidget->updateMenuAccel();
}


/* Behandlung des Menuepunkts: Options->Toolbar */
void TopLevel::toggleToolbar( void )
{
  /* Toolbar-Status show/hide togglen */
  enableToolBar();
  /* Variable des Status aucht togglen und Menuepunk
     dementsprechend markieren/demarkieren */
  showToolbar = (showToolbar) ? FALSE : TRUE;
  MenuBar->setItemChecked( toolbarID, showToolbar );
}


/* Behandlung des Menuepunkts: Options->Statusbar */
void TopLevel::toggleStatusbar( void )
{
  /* Statusbar-Status show/hide togglen */
  enableStatusBar();
  /* Variable des Status togglen und Menuepunkt
     dementsprechend markieren */
  showStatusbar = (showStatusbar) ? FALSE : TRUE;
  MenuBar->setItemChecked( statusbarID, showStatusbar );
}


/* Behandlung des Menuepunkts: Help->About */
/* Anzeigen der About-box */
void TopLevel::about( void )
{
  QString help;

  // VERSION wird vom Makefile gesetzt
  help.sprintf("KHeisePro (ehem. KHeise) v%s \n\n"
	       "(C) 1998, 1999, 2000 Alexander Bednarz & Martin Habbecke\n"
	       "<A.Bednarz@gmx.de>  <M.Habbecke@gmx.de>", VERSION);
  QMessageBox::about( this, "ber...", help );
}



/* Behandlung des Menuepukts: Help->Hilfe */
/* Starten der KDE-Hilfe */
void TopLevel::help( void )
{
   kapp->invokeHTMLHelp( "","" );
}



/* Behandlung eins Klicks auf die Titelzeile
   der Listbox von FoundList */
void TopLevel::handleHeaderClicked( int col )
{
  // cout << "click auf spalte: " << col << endl;
  
  if( !actFound )
    return;

  Found::SortType prevSort = actFound->getSortType();
  Found::SortType newSort = Found::TITEL_UP;
  
  switch( col )
    {
    case 0:
      /* Sortierung nach Uebereinstimmungs-Rate */
      if( prevSort == Found::RATING_UP )
	newSort = Found::RATING_DOWN;
      else
	newSort = Found::RATING_UP;
      break;
    case 1:
      /* Sortierung nach Titeln */
      if( prevSort == Found::TITEL_UP )
	newSort = Found::TITEL_DOWN;
      else
	newSort = Found::TITEL_UP;
      break;
    case 2:
      /* Sortierung nach Untertiteln gibts nicht */
      return;
      break;
    case 3:
      /* Sortierung nach Zeitschriftentyp */
      if( prevSort == Found::TYP_UP )
	newSort = Found::TYP_DOWN;
      else
	newSort = Found::TYP_UP;
      break;
    case 4:
      /* Sortierung nach Ausgabe (Nummer) */
      if( prevSort == Found::AUSG_UP )
	newSort = Found::AUSG_DOWN;
      else
	newSort = Found::AUSG_UP;
      break;
    case 5:
      /* Sortierung nach Jahrgang */
      if( prevSort == Found::JAHR_UP )
	newSort = Found::JAHR_DOWN;
      else
	newSort = Found::JAHR_UP;
      break;
   
    }
  /* Neuen Sortiertyp setzen, Liste neu sortieren
     und FoundList damit fuellen */
  //   found->setSortType( newSort );
  actFound->sort( newSort );
  mainWidget->fillList( actFound );
}


/** Slot zum setzen eines neuen Entry * in das
    ShowEntry-Widget, wenn sich der Current der
    Listbox von FoundList geaendert hat */
void TopLevel::setNewEntryInShowEntry( int index )
{
  // cout << "viewing new entry index: " << index << endl;
  /* LeseIndex des Found auf neuen Index setzen */
  actFound->setIndex( index );
  /* zugehoerigen Eintrag im ShowEntry anzeigen */
  mainWidget->updateShowEntry( actFound->get() );

  /* Muss der neue Entry auch ins Clipboard kopiert werden ? */
  if( optionsDialog->autoCopyToClipboard() )
    copyToClipboard();
      
}




/* Slot: Klick auf einen der Tabs im TabBar */
void TopLevel::setNewTab( int newId )
{
  int num;  // Anzahl der Eintraege des aktuellen Found

  if( newId == 0 )
    {
      actFound = found;
      /* Menuepunkte der Vormerkung richtig setzen */
      if( found == 0 )
	{
	  m_Search->setItemEnabled( preAllID, false );
	  m_Search->setItemEnabled( preMarkedID, false );
	  num = 0;
	}
      else
	{
	  m_Search->setItemEnabled( preAllID, true );
	  m_Search->setItemEnabled( preMarkedID, true );
	  num = found->count();
	}
      m_Search->setItemEnabled( remAllID, false );
      m_Search->setItemEnabled( remMarkedID, false );
    }
  else
    {
      actFound = markedFound;
      /* Menuepunkte der Vormerkung richtig setzen */
      m_Search->setItemEnabled( preAllID, false );
      m_Search->setItemEnabled( preMarkedID, false );
      if( (num = markedFound->count()) == 0 )
	{
	  m_Search->setItemEnabled( remAllID, false );
	  m_Search->setItemEnabled( remMarkedID, false );
	}
      else
	{
	  m_Search->setItemEnabled( remAllID, true );
	  m_Search->setItemEnabled( remMarkedID, true );
	}
    }
  /* Statusbar aktualisieren */
  QString hlp( "Eintrge: " );
  hlp.append( QString().setNum( num ) );
  StatusBar->changeItem( (const char *) hlp, 1 );  
  
  mainWidget->fillList( actFound );
}



/* Behandlung des KontextMenue-Punktes: Alle Eintraege vormerken */
void TopLevel::addAllToPreselect( void )
{
  for( int i = 0; i < found->count(); i++ )
    {
      found->setIndex( i );      
      markedFound->append( *found->get() );
    }
  // Jetzt kann man auch die Vorgemerkten Eintraege drucken
  m_File->setItemEnabled( printPreID, true );
  // Und ins Clipboard kopieren
  m_File->setItemEnabled( exportPreID, true );
}


/* Behandlung des KontextMenue-Punktes: Markierte Eintraege vormerken */
void TopLevel::addMarkedToPreselect( void )
{
  QList<int> markList = mainWidget->getMarkedItems();
  markList.first();

  /* Schleife ueber die Anzahl der enthaltenen Elemente */
  while( markList.current() != 0 )
    {
      found->setIndex( *markList.current() );
      /* Entry an markedFound anhaengen */
      markedFound->append( *found->get() );
      markList.next();
    }
  // Jetzt kann man auch die Vorgemerkten Eintraege drucken
  m_File->setItemEnabled( printPreID, true );
  // Und ins Clipboard kopieren
  m_File->setItemEnabled( exportPreID, true );
}



/* Behandlung des KontextMenue-Punktes: Alle Eintraege aus Vormerkung loeschen */
void TopLevel::removeAllFromPreselect( void )
{
  /* ListBox loeschen */
  mainWidget->fillList( 0 );
  
  /* Alten Act-Found loeschen und neuen (leeren) erstellen */
  delete markedFound;
  markedFound = new Found;
  actFound = markedFound;

  /* Menuepunkte disablen */
  m_Search->setItemEnabled( remAllID, false );
  m_Search->setItemEnabled( remMarkedID, false );
 
  m_File->setItemEnabled( printPreID, false );
  m_File->setItemEnabled( exportPreID, false );

  /* Statusbar richtig setzen */
  StatusBar->changeItem( "Eintrge: 0", 1 );
}


  
/* Behandlung des KontextMenue-Punktes: Markierte Eintraege aus Vormerkung loeschen */
void TopLevel::removeMarkedFromPreselect( void )
{
  QList<int> markList = mainWidget->getMarkedItems();
  markList.first();

  Found *newFound = new Found;

  /* Schleife ueber alle Elemente des actFound */
  for( int i = 0; i < markedFound->count(); i++ )
    {
      /* Ist der Index des aktuellen Elements in der
	 Liste der Markierten enthalten ? */
      if( (markList.current() == 0) || (*markList.current() != i) )
	{
	  /* Wenn nicht, dann soll das aktuelle Element nicht
	     geloescht werden, muss also uebernommen werden */
	  markedFound->setIndex( i );
	  newFound->append( *markedFound->get() );
	}
      else
	{
	  markList.next();
	}
    }
  
  /* Die Entries, die uebernommen werden sollen, stehen jetzt
     in newFound */
  delete markedFound;
  markedFound = newFound;

  /* Wenn kein Eintrag mehr vorgemerkt ist, dann muss der
     Menuepunkt zum drucken der vorgem. Eintr. gesperrt werden,
     natuerlich auch der Eintrag des ASCII-Exp. */
  if( markedFound->count() == 0 )
    {
      m_File->setItemEnabled( printPreID, false );
      m_File->setItemEnabled( exportPreID, false );
    }

  /* Aenderungen sichtbar machen */
  setNewTab( 1 );
}



/* Funktion zum exportieren eines Found als ASCII-Text in
   die Datei QFile file 
   Return-Wert: 0 - Ok
                1 - cancel
                2 - error  */
int TopLevel::exportFoundASCII( Found *vektor )
{
  const Entry *actEntry;
  QString line;

  /* Found auf Anfang setzen */
  vektor->setIndex( 0 );

  /* Datei fuer den Export oeffnen */
  QString filename = KFileDialog::getSaveFileName();
  if( filename.isEmpty() )
    return 1;

  /* Versuchen, diese Datei zu oeffnen */
  QFile outfile( filename );
  if( !outfile.open( IO_WriteOnly ) )
    return 2;

  /* Datensaetze exportieren */
  do {
    actEntry = vektor->get();
    line =  "Titel:       ";
    line += actEntry->Titel();
    line += "\n";
    line += "Untertitel:  ";
    line += actEntry->Untertitel();
    line += "\n";
    line += "Autor:       ";
    line += actEntry->Autor();
    line += "\n";
    line += "Redakteur:   ";
    line += actEntry->Redakteur();
    line += "\n";
    line += "Seite:       ";
    line += actEntry->Seite();
    line += "\n";
    line += "Ausgabe:     ";
    line += actEntry->Ausgabe();
    line += "/";
    line += actEntry->Jahrgang();
    line += "\n";
    line += "Zeitschrift: ";
    switch( actEntry->Zeitschrift() )
      {
      case (Entry::CT):
	line += "c't\n";
	break;
      case (Entry::IX):
	line += "iX\n";
	break;
      case (Entry::EL):
	line += "El\n";
	break;
      case (Entry::GW):
	line += "Gw\n";
	break;
      }
    line += "Verweise:    ";
    line += actEntry->Querverweise();
    line += "\n\n";

    outfile.writeBlock( line, line.length() );

  } while( vektor->next() );

  outfile.close();
  return 0;
}



/* Funktion zum Drucken eines Found-Vektors */
void TopLevel::printFound( Found *vektor )
{
  /* Index der gefundenen Eintraege auf 0 setzen */
  vektor->setIndex( 0 );

  /* Paint-Device erstellen */
  QPainter p;
  
  /* Fonts und FontMetrics anfordern */
  QFont fett( optionsDialog->getPrinterFont(), 
	      optionsDialog->getPrinterFontSize(), QFont::Bold );
  QFont normal( optionsDialog->getPrinterFont(), 
		optionsDialog->getPrinterFontSize() );

  QFont numFont( "times", 11, QFont::Bold );




  /* Rahmen in X- und Y-Richtung */
  int xFrame = 50, yFrame = 50;

  /* Aktueller Eintrag */
  const Entry *Item;
  char *Titel, *Untertitel, *Autor, *Querverweise;
  const char *Redakteur, *Seite, *Ausg, *Zeitschrift=0;
  char Ausgabe[20], pageNum[10], entryNum[10];


  /* Auf den Drucker zeichnen */
  p.begin( printer );

  /* FontMetrics anlegen */
  p.setFont( fett );
  QFontMetrics m_fett = p.fontMetrics();
  int space = m_fett.width( 'i' );
  int height = m_fett.height();

  p.setFont( normal );
  QFontMetrics m_normal = p.fontMetrics();
  p.setFont( numFont );
  QFontMetrics m_num = p.fontMetrics();

  int xMax = p.window().width();
  int yMax = p.window().height();


  /* Variablen der Koordinatenpaare der Beschreibungsfelder */
  int dTitel[2], dUntertitel[2], dAutor[2], dRedakteur[2], dSeite[2];
  int dAusgabe[2], dZeitschrift[2], dVerweise[2], dLfd[2];
  /* Variablen der Koordinatenpaare der Feldinhalte */
  int fTitel[2], fUntertitel[2], fAutor[2], fRedakteur[2], fSeite[2];
  int fAusgabe[2], fZeitschrift[2], fVerweise[2];
  int freespace;  // Variable des freien Raumes zwischen Ausg. und Zeitschr.

  /* Breiten der Beschreibungs-Woerter */
  p.setFont( fett );
  int wRedakteur = m_fett.width( "Redakteur:" );
  int wAusgabe = m_fett.width( "Ausgabe:" );
  int wZeitschrift = m_fett.width( "Zeitschrift:" );
  int fett_descent = m_fett.descent();
  int fett_ascent = m_fett.ascent();

  int yTop = 0, yGuess = 0;
  bool endreached = false;
  int pagecount = 1;
  int entrycount = 1;

  QRect originalView = p.viewport();


  yTop = yFrame + fett_ascent;
  
  /* Schleife ueber alle Eintraege */
  while( !endreached )
    {
      /* Aktuellen Eintrag holen */
      Item = vektor->get();

      /* Inhalt daraus auslesen */
      /* Kopiene anlegen, da die Strings evtl. abgeschnitten
	 und mit "..." aufgefuellt werden */
      Titel = new char[ strlen(Item->Titel()) + 1 ];
      strcpy( Titel, Item->Titel() );

      Untertitel = new char[ strlen(Item->Untertitel()) + 1 ];
      strcpy( Untertitel, Item->Untertitel() );

      Autor = new char[ strlen(Item->Autor()) + 1];
      strcpy( Autor, Item->Autor() );

      Redakteur = Item->Redakteur();
      Seite = Item->Seite();
      Ausg = Item->Ausgabe();

      Querverweise = new char[ strlen(Item->Querverweise()) + 1];
      strcpy( Querverweise, Item->Querverweise() );

      /* Zeitschrifen-Typ aufbereiten */
      switch( Item->Zeitschrift() )
	{
	case (Entry::CT):
	  Zeitschrift = "c't";
	  break;
	case (Entry::IX):
	  Zeitschrift = "iX";
	  break;
	case (Entry::EL):
	  Zeitschrift = "Elrad";
	  break;
	case (Entry::GW):
	  Zeitschrift = "Gateway";
	  break;
	}     
      Ausgabe[0] = 0;
      strcat( Ausgabe, Ausg );
      strcat( Ausgabe, " / " );
      strcat( Ausgabe, Item->Jahrgang() );
      
      /* Lfd-Nummer */
      sprintf( entryNum, "%d:", entrycount++ );
      
      

      /* Berechnung der Koordinaten der einzelnen 
	 auszugebenden Felder */
      dLfd[0] = xFrame;
      dLfd[1] = 0;
      
      p.setFont( fett );
      dTitel[0] = dLfd[0] + m_fett.width( entryNum ) + space;
      dTitel[1] = 0;
      fTitel[0] = dTitel[0] + space + wZeitschrift;
      fTitel[1] = dTitel[1];
      
      dUntertitel[0] = dTitel[0];
      if( strlen( Titel ) == 0 )
	dUntertitel[1] = dTitel[1];
      else
	dUntertitel[1] = dTitel[1] + height;
      fUntertitel[0] = fTitel[0];
      fUntertitel[1] = dUntertitel[1];
      
      dAutor[0] = dTitel[0];
      if( strlen( Untertitel ) == 0 )
	dAutor[1] = dUntertitel[1];
      else
	dAutor[1] = dUntertitel[1] + height;
      fAutor[0] = fTitel[0];
      fAutor[1] = dAutor[1];
      
      p.setFont( normal );
      freespace = xMax - xFrame - ( fAutor[0] + m_normal.width( Autor ) + space );
      if( freespace < ( wRedakteur + m_normal.width( Redakteur ) + space ) )
	{
	  /* Redakteur muss in eine neue Zeile */
	  dRedakteur[0] = dTitel[0];
	  dRedakteur[1] = dAutor[1] + height;
	  fRedakteur[0] = fTitel[0];
	  fRedakteur[1] = dRedakteur[1];
	}
      else
	{
	  /* Redakteur passt in die aktuelle Zeile */
	  if( strlen(Autor) )
	    {
	      freespace -= wRedakteur + space + m_normal.width( Redakteur );
	      dRedakteur[0] = fAutor[0] + m_normal.width( Autor ) + space + freespace / 2;
	      dRedakteur[1] = dAutor[1];
	      fRedakteur[0] = dRedakteur[0] + wRedakteur + space;
	      fRedakteur[1] = dRedakteur[1];
	    }
	  else
	    {
	      dRedakteur[0] = dTitel[0];
	      dRedakteur[1] = dAutor[1];
	      fRedakteur[0] = fTitel[0];
	      fRedakteur[1] = fAutor[1];
	    } 
	}

      
      dSeite[0] = dTitel[0];
      dSeite[1] = dRedakteur[1] + height;
      fSeite[0] = fTitel[0];
      fSeite[1] = dSeite[1];
      
      freespace = xMax - xFrame - ( fSeite[0] + m_normal.width( Seite ) + space );
      if( freespace < ( wAusgabe + wZeitschrift +
			m_normal.width( Ausgabe ) + m_normal.width( Zeitschrift ) + 3 * space ) )
	{
	  /* Zeitschrift muss in eine neue Zeile */
	  freespace -= wAusgabe + space + m_normal.width( Ausgabe );
	  dAusgabe[0] = fSeite[0] + m_normal.width( Seite ) + space + freespace / 2;
	  dAusgabe[1] = dSeite[1];
	  fAusgabe[0] = dAusgabe[0] + wAusgabe + space;
	  fAusgabe[1] = dAusgabe[1];
	  dZeitschrift[0] = dTitel[0];
	  dZeitschrift[1] = dSeite[1] + height;
	  fZeitschrift[0] = fTitel[0];
	  fZeitschrift[1] = dZeitschrift[1];
	}
      else
	{
	  /* Zeitschrift passt in die gleiche Zeile */
	  freespace -= wAusgabe + space + m_normal.width( Ausgabe ) +
	    space + wZeitschrift + space + m_normal.width( Zeitschrift );
	  dAusgabe[0] = fSeite[0] + m_normal.width( Seite ) + space + freespace / 3;
	  dAusgabe[1] = dSeite[1];
	  fAusgabe[0] = dAusgabe[0] + wAusgabe + space;
	  fAusgabe[1] = dAusgabe[1];
	  dZeitschrift[0] = fAusgabe[0] + m_normal.width( Ausgabe ) + space + freespace / 3;
	  dZeitschrift[1] = dSeite[1];
	  fZeitschrift[0] = dZeitschrift[0] + wZeitschrift + space;
	  fZeitschrift[1] = dZeitschrift[1];
	}
      
      dVerweise[0] = dTitel[0];
      dVerweise[1] = dZeitschrift[1] + height;
      fVerweise[0] = fTitel[0];
      fVerweise[1] = dVerweise[1];
      
      if( strlen( Querverweise ) == 0 )
	yGuess = dVerweise[1] + yTop - height + fett_descent;
      else
	yGuess = dVerweise[1] + yTop + fett_descent;
     

      /* Passt der Eintrag noch auf diese Seite ?? */
      p.setFont( numFont );
      if(  yGuess > yMax - yFrame - m_num.height() - 5 )
	{

	  /* Er passt nicht... */
	  
	  /* Am Ende einer Seite die Fusszeile mit Seiten-Nummer ausgeben */
	  p.setViewport( originalView );

	  p.drawLine( xFrame, yMax - yFrame - m_num.height(), 
		      xMax - xFrame, yMax - yFrame - m_num.height() );
	  sprintf( pageNum, "%d", pagecount++ );
	  p.drawText( xFrame, yMax - yFrame - m_num.height() + 4, 
		      xMax - 2*xFrame, m_num.height() + 2, AlignHCenter, pageNum );
	  
	  printer->newPage();
	  yTop = yFrame + fett_ascent;

	  //cout << "*** Page is full" << endl;
	  /* jetzt  passts wieder */
	}
      
      // cout << Titel << endl;
      
      /* Viewport auf die neuen Werte fuer den aktuellen
	 Eintrag setzen */
      p.setViewport( 0, yTop, xMax, yMax );

      /* Felder auf die maximale Groesse kuerzen */
      p.setFont( normal );
      shrinkToFit( Titel, xMax - xFrame - fTitel[0], m_normal);	
      shrinkToFit( Untertitel, xMax - xFrame - fUntertitel[0], m_normal );
      shrinkToFit( Autor, xMax - xFrame - fAutor[0], m_normal );
      shrinkToFit( Querverweise, xMax - xFrame - fVerweise[0], m_normal );

      /* Lfd-Nummer drucken */
      p.setFont( fett );
      p.drawText( dLfd[0], dLfd[1], entryNum ); 
      
      /* Aktuellen Eintrag auf diese Seite drucken */
      /* Nur wenn ein Titel gegeben ist, diesen auch wirklich drucken */
      /* => Vermeidung von leeren Zeilen */
      if( strlen(Titel) ) {
	p.setFont( fett );
	p.drawText( dTitel[0], dTitel[1], "Titel:" );
	p.setFont( normal );
	p.drawText( fTitel[0], fTitel[1], Titel );
      }

      /* Existiert Untertitel ?? */
      if( strlen(Untertitel) ) {
	p.setFont( fett );
	p.drawText( dUntertitel[0], dUntertitel[1], "Untertitel:" );
	p.setFont( normal );
	p.drawText( fUntertitel[0], fUntertitel[1], Untertitel ); 
      }
	
      /* Existiert Autor ?? */
      if( strlen(Autor) ) {
	p.setFont( fett );
	p.drawText( dAutor[0], dAutor[1], "Autor(en):" );	
	p.setFont( normal );
	p.drawText( fAutor[0], fAutor[1], Autor );
      }
	
      /* Existiert Redakteur ?? */
      if( strlen(Redakteur) ) {
	p.setFont( fett );
	p.drawText( dRedakteur[0], dRedakteur[1], "Redakteur:" );
	p.setFont( normal );
	p.drawText( fRedakteur[0], fRedakteur[1], Redakteur );
      }
      
	
      /* Felder Seite, Ausgabe und Zeitschrift sind immer belegt,
	 muessen also nicht ueberprueft werden */
      p.setFont( fett );
      p.drawText( dSeite[0], dSeite[1], "Seite:" );
      p.drawText( dAusgabe[0], dAusgabe[1], "Ausgabe:" );
      p.drawText( dZeitschrift[0], dZeitschrift[1], "Zeitschrift:" );
      p.setFont( normal );
      p.drawText( fSeite[0], fSeite[1], Seite );
      p.drawText( fAusgabe[0], fAusgabe[1], Ausgabe );
      p.drawText( fZeitschrift[0], fZeitschrift[1], Zeitschrift );
      yTop += height;
      
      /* Existieren Querverweise ?? */
      if( strlen(Querverweise) ) {
	p.setFont( fett );
	p.drawText( dVerweise[0], dVerweise[1], "Verweise:" );
	p.setFont( normal );
	p.drawText( fVerweise[0], fVerweise[1], Querverweise );
      }
	
      /* Versuchen, naechsten Eintrag zu holen */
      if( !vektor->next() )
	/* Klappt das nicht, dann flag auf Ende setzen */
	endreached = true;
      
      /* Anfang des naechsten Eintrags */
      if( strlen(Querverweise) )
	yTop += dVerweise[1] + height;
      else
	yTop += dVerweise[1];
      
      
      /* Die Strings des gedruckten Eintrags wieder loeschen */
      delete [] Titel;
      delete [] Untertitel;
      delete [] Autor;
      delete [] Querverweise;
      
    }

  /* Am Ende auf die letzte Seite noch die Fusszeile drucken */
  p.setFont( numFont );
  p.setViewport( originalView );
  p.drawLine( xFrame, yMax - yFrame - m_num.height(), 
	      xMax - xFrame, yMax - yFrame - m_num.height() );
  sprintf( pageNum, "%d", pagecount++ );
  p.drawText( xFrame, yMax - yFrame - m_num.height() + 4, 
	      xMax - 2*xFrame, m_num.height() + 2, AlignHCenter, pageNum );


  /* Drucken fertig */
  // cout << "*** Printing ready" << endl;
  p.end();
  
}



/* Funktion zum "schrumpfen" eines Strings, bis er
   in die gewuenschte Breite passt */
void TopLevel::shrinkToFit( char *str, int width, QFontMetrics &font )
{
  int len = strlen( str );
  if( font.width( str ) > width )
    {
      while( font.width( str, len ) > width )
	str[len--] = 0;
      
      strcpy( str + len - 2, "..." );
    }
}



/* Behandlung eines Klicks auf die CheckBox
   "Ct-Rom unterstuetzen" im OptionsDialog */
void TopLevel::receiveSupportCtrom( bool haveIt )
{
  // Variable in MainWidget setzen
  mainWidget->setHaveCtrom( haveIt );

  // Button vom MainWidget enablen / disablen
  mainWidget->setOpenButtonEnabled( actFound && actFound->count() && haveIt );
}



/* Oeffnen des aktuellen Artikels auf der 
   Ctrom mit dem KFM */
void TopLevel::openArticle( void )
{
  /* Aktueller Eintrag */
  const Entry *actEntry = actFound->get();

  QDir *checkCD = new QDir();
  checkCD->setSorting( QDir::Name );
  checkCD->setFilter( QDir::Dirs );
  

  /* Versuche, ins Verzeichnis "/html" zu wechseln */
  QString checkPath( optionsDialog->getMountPoint() );
  checkPath.append( "/html" );
  checkPath = QDir::cleanDirPath( checkPath );
  if( !checkCD->cd( checkPath, true ) )
    {
      QString helpme;
      helpme.sprintf( "Das Verzeichnis \"%s\" ist nicht lesbar.\nFalscher Mount-Point angegeben\n"
		      "oder c't-Rom nicht eingelegt / gemountet ?", 
		    (const char *) checkPath );
      QMessageBox::information( this, "KHeisePro", helpme );
      return;
    }


  /* Versuche jetzt, in das Verzeichnis /html/Jarhgang/Ausgabe
     zu wechseln */
  checkPath.append( "/" );
  checkPath.append( QString( actEntry->Jahrgang() ).rightJustify( 2, '0' ) );
  checkPath.append( "/" );
  checkPath.append( QString( actEntry->Ausgabe() ).rightJustify( 2, '0' ) );

  if( !checkCD->cd( checkPath, true ) )
    {
      /* Es gibt das Verzeichnis /html/Jahrgang/Ausgabe nicht */
      QString helpme;
      helpme.sprintf( "Das Verzeichnis \"%s\" \nist nicht lesbar. Falsche c't-Rom eingelegt ?", 
		      (const char * ) checkPath );
      QMessageBox::information( this, "KHeisePro", helpme );
      return;
    }


  /* Das Verzeichnis /html/Jahrgang/Ausgabe ist vorhanden */
  //cout << "Helper: " << (*checkCD)[2] << endl;

  /* Pfad fuer den Artikel sichern, falls der Pfad
     existiert */
  QString article = checkPath.copy();

  checkPath.append( "/" );
  checkPath.append( (*checkCD)[2] );
  checkPath.append( "/art.htm" );
  
  /* Teste, ob es ein art.htm im ersten Seiten-Verzeichnis gibt */
  if( !QFile( checkPath ).exists() )
    {
      /* Es gibt kein art.htm => es ist nicht die c't-Rom des aktuellen
	 Jahrgangs */
      QString helpme;
      helpme.sprintf( "Keine Artikel im aktuellen Jahrgang gefunden.\nFalsche c't-Rom eingelegt ?", 
		      (const char * ) checkPath );
      QMessageBox::information( this, "KHeisePro", helpme );
      return;
    }
  

  /* ab hier sollte jetzt die richtige c't-Rom erreichbar sein */

  article.append( "/" );
  QString pageNum( actEntry->Seite() );


  /* Seiten-Nr. herunterzaehlen (max. 10 Seiten) bis, bis ein 
     Artikel gefunden wurde (wichtig fuer "Hotline" eintraege der c't) */
  int count = 0;
  QString checkArt;
  bool articleFound = false;

  do
    {
      // Seiten-Nummer in den Pfad einbauen
      checkArt = article.copy();
      checkArt.append( pageNum.rightJustify( 3, '0' ) );
      checkArt.append( "/art.htm" );
      //cout << "Trying: " << checkArt << endl;
      articleFound = ( QFile( checkArt ).exists() );

      // Seiten-Nummer herunterzaehlen
      pageNum.setNum( pageNum.toUInt() - 1 );
      count++;
    } while( !articleFound && count < 10 );
    

  if( !articleFound )
    {
      QString helpme;
      helpme.sprintf( "Der gewuenschte Artikel konnte auf der\nc't-Rom nicht gefunden werden.", 
		      (const char * ) checkPath );
      QMessageBox::information( this, "KHeisePro", helpme );
      return;
    }
      

  /* Jetzt ist der Artikel lesbar */


  //cout << "Opending: " << checkArt << endl;

  /* Mit welchem Browser soll der Artikel geoeffnet werden ? */
  QString browser = optionsDialog->getBrowser();
  QString commandline;

  if( browser == "kfm" )
    {
      if( kfm == 0 )
	{
	  kfm = new KFM;
	  kfm->allowKFMRestart( true );
	}
      checkArt.prepend( "file:" );
      kfm->openURL( QDir::cleanDirPath( checkArt ) );
    }
  else
    {
      commandline = browser.replace( QRegExp("%s"), (const char *)QDir::cleanDirPath( checkArt ) );
      cout << "Aufruf: " << commandline << endl;

      system( commandline );
    }
}


/* Behandlung eines Klicks auf den Toolbar-Button "Copy":
   Kopieren aller markierter Eintraege in das
   X-clipboard */
void TopLevel::copyToClipboard( void )
{
  QString clipText;
  const Entry *actEntry;
  QList<int> markList = mainWidget->getMarkedItems();

  Found *clipVek = new Found;
  int *index = markList.first();
  bool first = true;

  /* gibt es markierte Elemente ? */
  if( !markList.isEmpty() )
    {
      do {
	actFound->setIndex( *index );
	clipVek->append( *actFound->get() );
      } while( (index = markList.next()) );
    
      /* Found-Vektor "clipVek" ins Clipboard kopieren */
      QClipboard *cb = KApplication::clipboard();
      clipVek->setIndex( 0 );

      /* Eintraege des clipVek lesen und in den String clipText speichern */
      do {
	actEntry = clipVek->get();

	if( optionsDialog->titelToClipboard() )
	  {
	    if( first ) first = false;
	    else clipText += optionsDialog->separatorString();
	    clipText += actEntry->Titel();
	  }
	if( optionsDialog->untertitelToClipboard() )
	  {
	    if( first ) first = false;
	    else clipText += optionsDialog->separatorString();
	    clipText += actEntry->Untertitel();
	  }
	if( optionsDialog->autorToClipboard() )
	  {
	    if( first ) first = false;
	    else clipText += optionsDialog->separatorString();
	    clipText += actEntry->Autor();
	  }
	if( optionsDialog->redakteurToClipboard() )
	  {
	    if( first ) first = false;
	    else clipText += optionsDialog->separatorString();
	    clipText += actEntry->Redakteur();
	  }
	if( optionsDialog->seiteToClipboard() )
	  {
	    if( first ) first = false;
	    else clipText += optionsDialog->separatorString();
	    clipText += actEntry->Seite();
	  }
	if( optionsDialog->ausgabeToClipboard() )
	  {
	    if( first ) first = false;
	    else clipText += optionsDialog->separatorString();
	    clipText += actEntry->Ausgabe();
	    clipText += "/";
	    clipText += actEntry->Jahrgang();
	  }
	if( optionsDialog->zeitschriftToClipboard() )
	  {
	    if( first ) first = false;
	    else clipText += optionsDialog->separatorString();

	    /* Zeitschriftentyp aufbereiten */
	    switch( actEntry->Zeitschrift() )
	      {
	      case (Entry::CT):
		clipText += "c't";
		break;
	      case (Entry::IX):
		clipText += "iX";
		break;
	      case (Entry::EL):
		clipText += "El";
		break;
	      case (Entry::GW):
		clipText += "Gw";
		break;
	      }
	  }
	if( optionsDialog->verweiseToClipboard() )
	  {
	    if( first ) first = false;
	    else clipText += optionsDialog->separatorString();
	    clipText += actEntry->Querverweise();
	  }

	clipText += "\n";
	first = true;
      } while( clipVek->next() );
      cb->setText( (const char *) clipText );

    }
  delete clipVek;  

}

#include "toplevel.moc"
